@@ -38,6 +38,7 @@ dependencies.
 
 import sys
 import os
+import platform
 import numpy
 import scipy
 import qutip.settings
@@ -54,22 +55,26 @@ def about():
     print("Copyright (c) 2011 and later.")
     print("Paul D. Nation & Robert J. Johansson")
     print('')
-    print("QuTiP Version:       " + qutip.__version__)
-    print("Numpy Version:       " + numpy.__version__)
-    print("Scipy Version:       " + scipy.__version__)
+    print("QuTiP Version:      ", qutip.__version__)
+    print("Numpy Version:      ", numpy.__version__)
+    print("Scipy Version:      ", scipy.__version__)
     try:
         import Cython
         cython_ver = Cython.__version__
     except:
         cython_ver = 'None'
-    print(("Cython Version:      " + cython_ver))
+    print("Cython Version:     ", cython_ver)
     try:
         import matplotlib
         matplotlib_ver = matplotlib.__version__
     except:
         matplotlib_ver = 'None'
-    print(("Matplotlib Version:  " + matplotlib_ver))
-    print('')
+    print("Matplotlib Version: ", matplotlib_ver)
+    print("Fortran mcsolver:   ", str(qutip.settings.fortran))
+    print("scikits.umfpack:    ", str(qutip.settings.umfpack))
+    print("Python Version:     ", '.'.join(map(str, sys.version_info[0:3])))
+    print("Platform Info:      ", platform.system(), "("+platform.machine()+")")
+    print()
 
 if __name__ == "__main__":
     about()
