@@ -1,40 +1,53 @@
-#This file is part of QuTiP.
+# This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    QuTiP is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#   (at your option) any later version.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    All rights reserved.
 #
-#    QuTiP is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#    Redistribution and use in source and binary forms, with or without
+#    modification, are permitted provided that the following conditions are
+#    met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with QuTiP.  If not, see <http://www.gnu.org/licenses/>.
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
 #
-# Copyright (C) 2011-2013, Paul D. Nation & Robert J. Johansson
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
 #
-###########################################################################
+#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
+#       of its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+###############################################################################
 
 import numpy as np
 import scipy.integrate
 
-from qutip.qobj import Qobj
-from qutip.superoperator import *
+from qutip.qobj import Qobj, isket
+from qutip.superoperator import spre, spost, vec2mat, mat2vec, vec2mat_index
 from qutip.expect import expect
-from qutip.states import *
-from qutip.odeoptions import Odeoptions
-from qutip.cyQ.spmatfuncs import cy_ode_rhs
-from qutip.odedata import Odedata
+from qutip.solver import Options
+from qutip.cy.spmatfuncs import cy_ode_rhs
+from qutip.solver import Result
 
 
-#------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Solve the Bloch-Redfield master equation
 #
 #
-def brmesolve(H, psi0, tlist, c_ops, e_ops=[], spectra_cb=[],
-              args={}, options=Odeoptions()):
+def brmesolve(H, psi0, tlist, a_ops, e_ops=[], spectra_cb=[],
+              args={}, options=Options()):
     """
     Solve the dynamics for the system using the Bloch-Redfeild master equation.
 
@@ -55,10 +68,10 @@ def brmesolve(H, psi0, tlist, c_ops, e_ops=[], spectra_cb=[],
     tlist : *list* / *array*
         List of times for :math:`t`.
 
-    c_ops : list of :class:`qutip.qobj`
-        List of collapse operators.
+    a_ops : list of :class:`qutip.qobj`
+        List of system operators that couple to bath degrees of freedom.
 
-    expt_ops : list of :class:`qutip.qobj` / callback function
+    e_ops : list of :class:`qutip.qobj` / callback function
         List of operators for which to evaluate expectation values.
 
     args : *dictionary*
@@ -71,25 +84,26 @@ def brmesolve(H, psi0, tlist, c_ops, e_ops=[], spectra_cb=[],
     Returns
     -------
 
-    output: :class:`qutip.odedata`
+    output: :class:`qutip.solver`
 
-        An instance of the class :class:`qutip.odedata`, which contains either
-        an *array* of expectation values for the times specified by `tlist`.
+        An instance of the class :class:`qutip.solver`, which contains either
+        a list of expectation values, for operators given in e_ops, or a list
+        of states for the times specified by `tlist`.
     """
 
-    if len(spectra_cb) == 0:
-        for n in range(len(c_ops)):
-            # add white noise callbacks if absent
-            spectra_cb.append(lambda w: 1.0)
+    if not spectra_cb:
+        # default to infinite temperature white noise
+        spectra_cb = [lambda w: 1.0 for _ in a_ops]
 
-    R, ekets = bloch_redfield_tensor(H, c_ops, spectra_cb)
+    R, ekets = bloch_redfield_tensor(H, a_ops, spectra_cb)
 
-    output = Odedata()
+    output = Result()
+    output.solver = "brmesolve"
     output.times = tlist
 
     results = bloch_redfield_solve(R, ekets, psi0, tlist, e_ops, options)
 
-    if len(e_ops):
+    if e_ops:
         output.expect = results
     else:
         output.states = results
@@ -97,7 +111,7 @@ def brmesolve(H, psi0, tlist, c_ops, e_ops=[], spectra_cb=[],
     return output
 
 
-#------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Evolution of the Bloch-Redfield master equation given the Bloch-Redfield
 # tensor.
 #
@@ -131,16 +145,15 @@ def bloch_redfield_solve(R, ekets, rho0, tlist, e_ops=[], options=None):
     Returns
     -------
 
-    output: :class:`qutip.odedata`
+    output: :class:`qutip.solver`
 
-        An instance of the class :class:`qutip.odedata`, which contains either
+        An instance of the class :class:`qutip.solver`, which contains either
         an *array* of expectation values for the times specified by `tlist`.
 
     """
 
     if options is None:
-        options = Odeoptions()
-        options.nsteps = 2500
+        options = Options()
 
     if options.tidy:
         R.tidyup()
@@ -155,63 +168,54 @@ def bloch_redfield_solve(R, ekets, rho0, tlist, e_ops=[], options=None):
     #
     # prepare output array
     #
-    n_e_ops = len(e_ops)
     n_tsteps = len(tlist)
     dt = tlist[1] - tlist[0]
-
-    if n_e_ops == 0:
-        result_list = []
-    else:
-        result_list = []
-        for op in e_ops:
-            if op.isherm and rho0.isherm:
-                result_list.append(np.zeros(n_tsteps))
-            else:
-                result_list.append(np.zeros(n_tsteps, dtype=complex))
+    result_list = []
 
     #
     # transform the initial density matrix and the e_ops opterators to the
     # eigenbasis
     #
-    if ekets is not None:
-        rho0 = rho0.transform(ekets)
-        for n in arange(len(e_ops)):
-            e_ops[n] = e_ops[n].transform(ekets, False)
+    rho_eb = rho0.transform(ekets)
+    e_eb_ops = [e.transform(ekets) for e in e_ops]
+
+    for e_eb in e_eb_ops:
+        result_list.append(np.zeros(n_tsteps, dtype=complex))
 
     #
     # setup integrator
     #
-    initial_vector = mat2vec(rho0.full())
+    initial_vector = mat2vec(rho_eb.full())
     r = scipy.integrate.ode(cy_ode_rhs)
     r.set_f_params(R.data.data, R.data.indices, R.data.indptr)
     r.set_integrator('zvode', method=options.method, order=options.order,
                      atol=options.atol, rtol=options.rtol,
-                     #nsteps=options.nsteps,
-                     #first_step=options.first_step, min_step=options.min_step,
-                     max_step=options.max_step)
+                     nsteps=options.nsteps, first_step=options.first_step,
+                     min_step=options.min_step, max_step=options.max_step)
     r.set_initial_value(initial_vector, tlist[0])
 
     #
     # start evolution
     #
-    rho = Qobj(rho0)
+    dt = np.diff(tlist)
+    for t_idx, _ in enumerate(tlist):
 
-    t_idx = 0
-    for t in tlist:
         if not r.successful():
             break
 
-        rho.data = vec2mat(r.y)
+        rho_eb.data = vec2mat(r.y)
 
-        # calculate all the expectation values, or output rho if no operators
-        if n_e_ops == 0:
-            result_list.append(Qobj(rho))
+        # calculate all the expectation values, or output rho_eb if no
+        # expectation value operators are given
+        if e_ops:
+            rho_eb_tmp = Qobj(rho_eb)
+            for m, e in enumerate(e_eb_ops):
+                result_list[m][t_idx] = expect(e, rho_eb_tmp)
         else:
-            for m in range(0, n_e_ops):
-                result_list[m][t_idx] = expect(e_ops[m], rho)
+            result_list.append(rho_eb.transform(ekets, True))
 
-        r.integrate(r.t + dt)
-        t_idx += 1
+        if t_idx < n_tsteps - 1:
+            r.integrate(r.t + dt[t_idx])
 
     return result_list
 
@@ -220,7 +224,7 @@ def bloch_redfield_solve(R, ekets, rho0, tlist, e_ops=[], options=None):
 # Functions for calculting the Bloch-Redfield tensor for a time-independent
 # system.
 #
-def bloch_redfield_tensor(H, c_ops, spectra_cb, use_secular=True):
+def bloch_redfield_tensor(H, a_ops, spectra_cb, use_secular=True):
     """
     Calculate the Bloch-Redfield tensor for a system given a set of operators
     and corresponding spectral functions that describes the system's coupling
@@ -232,8 +236,8 @@ def bloch_redfield_tensor(H, c_ops, spectra_cb, use_secular=True):
     H : :class:`qutip.qobj`
         System Hamiltonian.
 
-    c_ops : list of :class:`qutip.qobj`
-        List of collapse operators.
+    a_ops : list of :class:`qutip.qobj`
+        List of system operators that couple to the environment.
 
     spectra_cb : list of callback functions
         List of callback functions that evaluate the noise power spectrum
@@ -255,13 +259,21 @@ def bloch_redfield_tensor(H, c_ops, spectra_cb, use_secular=True):
 
     # Sanity checks for input parameters
     if not isinstance(H, Qobj):
-        raise "H must be a quantum object"
+        raise TypeError("H must be an instance of Qobj")
+
+    for a in a_ops:
+        if not isinstance(a, Qobj) or not a.isherm:
+            raise TypeError("Operators in a_ops must be Hermitian Qobj.")
+
+    # default spectrum
+    if not spectra_cb:
+        spectra_cb = [lambda w: 1.0 for _ in a_ops]
 
     # use the eigenbasis
     evals, ekets = H.eigenstates()
 
     N = len(evals)
-    K = len(c_ops)
+    K = len(a_ops)
     A = np.zeros((K, N, N), dtype=complex)  # TODO: use sparse here
     W = np.zeros((N, N))
 
@@ -271,8 +283,8 @@ def bloch_redfield_tensor(H, c_ops, spectra_cb, use_secular=True):
             W[m, n] = np.real(evals[m] - evals[n])
 
     for k in range(K):
-        #A[k,n,m] = c_ops[k].matrix_element(ekets[n], ekets[m])
-        A[k, :, :] = c_ops[k].transform(ekets).full()
+        # A[k,n,m] = a_ops[k].matrix_element(ekets[n], ekets[m])
+        A[k, :, :] = a_ops[k].transform(ekets).full()
 
     dw_min = abs(W[W.nonzero()]).min()
 
@@ -286,7 +298,7 @@ def bloch_redfield_tensor(H, c_ops, spectra_cb, use_secular=True):
             c, d = vec2mat_index(N, J)
 
             # unitary part: use spre and spost above, same as this:
-            # R[I,J] = -1j * W[a,b] * (a == c) * (b == d)
+            # R.data[I,J] = -1j * W[a,b] * (a == c) * (b == d)
 
             if use_secular is False or abs(W[a, b] - W[c, d]) < dw_min / 10.0:
 
@@ -295,8 +307,8 @@ def bloch_redfield_tensor(H, c_ops, spectra_cb, use_secular=True):
                     # for each operator coupling the system to the environment
 
                     R.data[I, J] += ((A[k, a, c] * A[k, d, b] / 2) *
-                                    (spectra_cb[k](W[c, a]) +
-                                     spectra_cb[k](W[d, b])))
+                                     (spectra_cb[k](W[c, a]) +
+                                      spectra_cb[k](W[d, b])))
                     s1 = s2 = 0
                     for n in range(N):
                         s1 += A[k, a, n] * A[k, n, c] * spectra_cb[k](W[c, n])
