@@ -1,32 +1,48 @@
-# This file is part of QuTiP.
+# This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    QuTiP is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    All rights reserved.
 #
-#    QuTiP is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#    Redistribution and use in source and binary forms, with or without
+#    modification, are permitted provided that the following conditions are
+#    met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with QuTiP.  If not, see <http://www.gnu.org/licenses/>.
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
 #
-# Copyright (C) 2011-2013, Paul D. Nation & Robert J. Johansson
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
 #
-###########################################################################
+#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
+#       of its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+###############################################################################
 """
 This module is a collection of random state and operator generators.
 The sparsity of the ouput Qobj's is controlled by varing the
 `density` parameter.
-
 """
 from scipy import arcsin, sqrt, pi
 import numpy as np
 import scipy.linalg as la
+from scipy.linalg.matfuncs import sqrtm
 import scipy.sparse as sp
 from qutip.qobj import *
+from qutip.operators import create, destroy, jmat
+from operator import add
 
 
 def rand_herm(N, density=0.75, dims=None):
@@ -62,6 +78,7 @@ def rand_herm(N, density=0.75, dims=None):
     Y = X.copy()
     Y.data = 1.0j * np.random.random(len(X.data)) - (0.5 + 0.5j)
     X = X + Y
+    X.sort_indices()
     X = Qobj(X)
     if dims:
         return Qobj((X + X.dag()) / 2.0, dims=dims, shape=[N, N])
@@ -94,6 +111,7 @@ def rand_unitary(N, density=0.75, dims=None):
     if dims:
         _check_dims(dims, N, N)
     U = (-1.0j * rand_herm(N, density)).expm()
+    U.data.sort_indices()
     if dims:
         return Qobj(U, dims=dims, shape=[N, N])
     else:
@@ -126,6 +144,7 @@ def rand_ket(N, density=1, dims=None):
     Y = X.copy()
     Y.data = 1.0j * np.random.random(len(X.data)) - (0.5 + 0.5j)
     X = X + Y
+    X.sort_indices()
     X = Qobj(X)
     if dims:
         return Qobj(X / X.norm(), dims=dims, shape=[N, 1])
@@ -141,7 +160,7 @@ def rand_dm(N, density=0.75, pure=False, dims=None):
     N : int
         Shape of output density matrix.
     density : float
-        Density etween [0,1] of output density matrix.
+        Density between [0,1] of output density matrix.
     dims : list
         Dimensions of quantum object.  Used for specifying
         tensor structure. Default is dims=[[N],[N]].
@@ -154,7 +173,7 @@ def rand_dm(N, density=0.75, pure=False, dims=None):
 
     Notes
     -----
-    For small density matricies, choosing a low density will result in an error
+    For small density matrices., choosing a low density will result in an error
     as no diagonal elements will be generated such that :math:`Tr(\\rho)=1`.
 
     """
@@ -176,12 +195,51 @@ def rand_dm(N, density=0.75, pure=False, dims=None):
         if tries >= 10:
             raise ValueError(
                 "Requested density is too low to generate density matrix.")
+    H.data.sort_indices()
     if dims:
         return Qobj(H / H.tr(), dims=dims, shape=[N, N])
     else:
         return Qobj(H / H.tr())
 
 
+def rand_kraus_map(N, dims=None):
+    """
+    Creates a random CPTP map on an N-dimensional Hilbert space in Kraus
+    form.
+
+    Parameters
+    ----------
+    N : int
+        Length of input/output density matrix.
+    dims : list
+        Dimensions of quantum object.  Used for specifying
+        tensor structure. Default is dims=[[N],[N]].
+
+
+    Returns
+    -------
+    oper_list : list of qobj
+        N^2 x N x N qobj operators.
+
+    """
+
+    if dims:
+        _check_dims(dims, N, N)
+
+    # Random unitary (Stinespring Dilation)
+    big_unitary = rand_unitary(N ** 3).data.todense()
+    orthog_cols = np.array(big_unitary[:, :N])
+    oper_list = np.reshape(orthog_cols, (N ** 2, N, N))
+    return list(map(lambda x: Qobj(inpt=x, dims=dims), oper_list))
+
+
+def rand_super(dim=5):
+    H = rand_herm(dim)
+    return propagator(H, np.random.rand(), [
+        create(dim), destroy(dim), jmat(float(dim - 1) / 2.0, 'z')
+    ])
+
+
 def _check_dims(dims, N1, N2):
     if len(dims) != 2:
         raise Exception("Qobj dimensions must be list of length 2.")
@@ -192,3 +250,7 @@ def _check_dims(dims, N1, N2):
         raise ValueError("Qobj dimensions must match matrix shape.")
     if len(dims[0]) != len(dims[1]):
         raise TypeError("Qobj dimension components must have same length.")
+
+# TRAILING IMPORTS
+# qutip.propagator depends on rand_dm, so we need to put this import last.
+from qutip.propagator import propagator
