@@ -1,21 +1,35 @@
-# This file is part of QuTiP.
+# This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    QuTiP is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    All rights reserved.
 #
-#    QuTiP is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#    Redistribution and use in source and binary forms, with or without
+#    modification, are permitted provided that the following conditions are
+#    met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with QuTiP.  If not, see <http://www.gnu.org/licenses/>.
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
 #
-# Copyright (C) 2011-2013, Paul D. Nation & Robert J. Johansson
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
 #
-###########################################################################
+#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
+#       of its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+###############################################################################
 
 import numpy as np
 import scipy
@@ -61,7 +75,7 @@ def test_compare_solvers_steadystate():
 
 
 def test_spectrum():
-    "correlation: compare spectrum obtained for eseries and fft methods"
+    "correlation: compare spectrum from eseries and fft methods"
 
     # use JC model
     N = 4
@@ -86,5 +100,31 @@ def test_spectrum():
 
     assert_(max(abs(spec1 - spec2)) < 1e-3)
 
+
+def test_spectrum():
+    "correlation: compare spectrum from eseries and pseudo-inverse methods"
+
+    # use JC model
+    N = 4
+    wc = wa = 1.0 * 2 * pi
+    g = 0.1 * 2 * pi
+    kappa = 0.75
+    gamma = 0.25
+    n_th = 0.01
+
+    a = tensor(destroy(N), qeye(2))
+    sm = tensor(qeye(N), destroy(2))
+    H = wc * a.dag() * a + wa * sm.dag() * sm + \
+        g * (a.dag() * sm + a * sm.dag())
+    c_ops = [sqrt(kappa * (1 + n_th)) * a,
+             sqrt(kappa * n_th) * a.dag(),
+             sqrt(gamma) * sm]
+
+    wlist = 2 * pi * np.linspace(0.5, 1.5, 100)
+    spec1 = spectrum_ss(H, wlist, c_ops, a.dag(), a)
+    spec2 = spectrum_pi(H, wlist, c_ops, a.dag(), a)
+
+    assert_(max(abs(spec1 - spec2)) < 1e-3)
+
 if __name__ == "__main__":
     run_module_suite()
